#!/usr/bin/ruby

require 'rice/irc'
require 'rice/drb'
include RICE::Command

URI  = 'druby://localhost:9876'
CHANNEL = '#ricetest'

DRb.start_service
irc = DRbObject.new(nil, URI)

puts "input message for #{CHANNEL}:"
irc.push privmsg CHANNEL, gets

