#!/usr/bin/ruby

require 'rice/irc'
require 'rice/proxy'

NICK = 'irb'
USER = ENV['USER'] || ENV['USERNAME'] || ENV['LOGNAME'] || NICK
REAL = 'rice/irb user'

CHANNEL = '#ricetest'

Thread.abort_on_exception = true

c = RICE::Connection.new('localhost', 9999)
proxy = RICE::Proxy.new(c, 'localhost', 8888)

c.start


